@@ -1,30 +1,33 @@
 """A module for deserializing data to Python objects."""
 
-# pylint: disable=unidiomatic-typecheck
 # pylint: disable=protected-access
 # pylint: disable=too-many-branches
-# pylint: disable=wildcard-import
 
 import enum
-import typing
-from typing import Any, Dict,  Optional
+import inspect
+from typing import Any, Annotated, TypeVar, cast, overload
 
 from deserialize.conversions import camel_case, pascal_case
-from deserialize.decorators import constructed, _call_constructed
-from deserialize.decorators import default, _get_default, _has_default
+from deserialize.custom_deserializable import CustomDeserializable
+from deserialize.decorators import (
+    constructed,
+    _call_constructed,
+)
+from deserialize.decorators import default
 from deserialize.decorators import (
     downcast_field,
-    _get_downcast_field,
     downcast_identifier,
     _get_downcast_class,
     allow_downcast_fallback,
-    _allows_downcast_fallback,
 )
-from deserialize.decorators import ignore, _should_ignore
-from deserialize.decorators import key, _get_key
-from deserialize.decorators import parser, _get_parser
-from deserialize.decorators import auto_snake, _uses_auto_snake
-from deserialize.decorators import allow_unhandled, _should_allow_unhandled
+from deserialize.decorators import ignore
+from deserialize.decorators import key
+from deserialize.decorators import parser
+from deserialize.decorators import auto_snake
+from deserialize.decorators import (
+    allow_unhandled,
+    _should_allow_unhandled,
+)
 
 from deserialize.exceptions import (
     DeserializeException,
@@ -32,53 +35,106 @@ from deserialize.exceptions import (
     UndefinedDowncastException,
     UnhandledFieldException,
 )
-from deserialize.type_checks import *
+from deserialize.raw_storage_mode import RawStorageMode
+from deserialize.type_checks import (
+    is_classvar,
+    is_union,
+    union_types,
+    is_typing_type,
+    is_list,
+    is_dict,
+    is_set,
+    is_tuple,
+    list_content_type,
+    dict_content_types,
+    set_content_type,
+    tuple_content_types,
+)
+from deserialize.metadata_cache import get_class_metadata
+from deserialize.field import Field
+
+# Type variable for deserialization
+T = TypeVar("T")
 
 
-class RawStorageMode(enum.Enum):
-    """The storage mode for the raw data on each object.
+# Public API - explicitly declare re-exports for type checkers
+__all__ = [
+    # Main function
+    "deserialize",
+    # Decorators
+    "constructed",
+    "default",
+    "downcast_field",
+    "downcast_identifier",
+    "allow_downcast_fallback",
+    "ignore",
+    "key",
+    "parser",
+    "auto_snake",
+    "allow_unhandled",
+    # Exceptions
+    "DeserializeException",
+    "InvalidBaseTypeException",
+    "UndefinedDowncastException",
+    "UnhandledFieldException",
+    # Enums
+    "RawStorageMode",
+    # Type checks
+    "is_classvar",
+    "is_union",
+    "union_types",
+    "is_typing_type",
+    "is_list",
+    "is_dict",
+    "is_set",
+    "is_tuple",
+    "list_content_type",
+    "dict_content_types",
+    "set_content_type",
+    "tuple_content_types",
+    # Field annotation support
+    "Field",
+    "Annotated",
+    # Custom deserialization protocol
+    "CustomDeserializable",
+    # Utilities
+    "camel_case",
+    "pascal_case",
+    "get_class_metadata",
+]
 
-    If a store mode is set, the data will be stored in the attribute named:
-    `__deserialize_raw__`
-    """
 
-    # Do not store the raw data at all
-    NONE = "none"
+@overload
+def deserialize(
+    class_reference: type[T],
+    data: Any,
+    *,
+    throw_on_unhandled: bool = False,
+    raw_storage_mode: RawStorageMode = RawStorageMode.NONE,
+) -> T: ...
 
-    # Only store the data on the root node
-    ROOT = "root"
 
-    # Store on all objects (WARNING: This can use a significant amount of memory)
-    ALL = "all"
-
-    def child_mode(self) -> "RawStorageMode":
-        """Determine the mode for child parsing.
-
-        When we move to the next child iteration, we need to change mode
-        in some cases. For instance, if we only store the root node, then we
-        need to set all the children to not be stored.
-
-        :raises Exception: If we get an unexpected storage mode
-
-        :returns: The child raw storage mode
-        """
-        if self == RawStorageMode.NONE:
-            return RawStorageMode.NONE
-
-        if self == RawStorageMode.ROOT:
-            return RawStorageMode.NONE
-
-        if self == RawStorageMode.ALL:
-            return RawStorageMode.ALL
-
-        raise DeserializeException(f"Unexpected raw storage mode: {self}")
+@overload
+def deserialize(
+    class_reference: Any,
+    data: Any,
+    *,
+    throw_on_unhandled: bool = False,
+    raw_storage_mode: RawStorageMode = RawStorageMode.NONE,
+) -> Any: ...
 
 
 # pylint: disable=function-redefined
-def deserialize(class_reference, data, *, throw_on_unhandled: bool = False, raw_storage_mode: RawStorageMode = RawStorageMode.NONE):  # type: ignore
+def deserialize(  # type: ignore
+    class_reference: type[T],
+    data: dict[Any, Any] | list[Any],
+    *,
+    throw_on_unhandled: bool = False,
+    raw_storage_mode: RawStorageMode = RawStorageMode.NONE,
+) -> T:
     """Deserialize data to a Python object."""
 
-    if not isinstance(data, dict) and not isinstance(data, list):
+    if not isinstance(data, dict) and not isinstance(data, list):  # type: ignore[unreachable]
         raise InvalidBaseTypeException(
             "Only lists and dictionaries are supported as base raw data types"
         )
@@ -102,8 +158,13 @@ def deserialize(class_reference, data, *, throw_on_unhandled: bool = False, raw_
 
 # pylint:disable=too-many-return-statements
 def _deserialize(
-    class_reference, data, debug_name, *, throw_on_unhandled: bool, raw_storage_mode: RawStorageMode
-):
+    class_reference: type[T],
+    data: Any,
+    debug_name: str,
+    *,
+    throw_on_unhandled: bool,
+    raw_storage_mode: RawStorageMode,
+) -> T:
     """Deserialize data to a Python object, but allow base types"""
 
     # In here we try and use some "heuristics" to deserialize. We have 2 main
@@ -122,7 +183,7 @@ def _deserialize(
     # then handle collection data, then any other types afterwards. That's not
     # set in stone though.
 
-    def finalize(value: Optional[Any]) -> Optional[Any]:
+    def finalize(value: T) -> T:
         """Run through any finalization steps before returning the value."""
 
         # Set raw data where applicable
@@ -133,17 +194,23 @@ def _deserialize(
 
         return value
 
-    if class_reference == Any:
-        return finalize(data)
+    if class_reference is Any:  # type: ignore[comparison-overlap]
+        return cast(T, data)
 
-    # Check if it's None (since things like Union[int, Optional[str]] become
+    if inspect.isclass(class_reference) and issubclass(class_reference, CustomDeserializable):
+        # If the class is a custom deserializable, we need to call the
+        # deserialize method on it
+        result = class_reference.deserialize(data)
+        return finalize(cast(T, result))
+
+    # Check if it's None (since things like Union[int, str | None] become
     # Union[int, str, None] so we end up iterating against it)
     if class_reference == type(None) and data is None:
-        return finalize(None)
+        return cast(T, None)
 
     if is_union(class_reference):
         valid_types = union_types(class_reference, debug_name)
-        exceptions = []
+        exceptions: list[str] = []
         for valid_type in valid_types:
             try:
                 return finalize(
@@ -184,7 +251,7 @@ def _deserialize(
         return finalize(
             _deserialize_dict(
                 class_reference,
-                data,
+                cast(dict[Any, Any], data),
                 debug_name,
                 throw_on_unhandled=throw_on_unhandled,
                 raw_storage_mode=raw_storage_mode,
@@ -192,10 +259,30 @@ def _deserialize(
         )
 
     if isinstance(data, list):
+        if is_set(class_reference):
+            return finalize(
+                _deserialize_set(
+                    class_reference,
+                    cast(list[Any], data),
+                    debug_name,
+                    throw_on_unhandled=throw_on_unhandled,
+                    raw_storage_mode=raw_storage_mode,
+                )
+            )
+        if is_tuple(class_reference):
+            return finalize(
+                _deserialize_tuple(
+                    class_reference,
+                    cast(list[Any], data),
+                    debug_name,
+                    throw_on_unhandled=throw_on_unhandled,
+                    raw_storage_mode=raw_storage_mode,
+                )
+            )
         return finalize(
             _deserialize_list(
                 class_reference,
-                data,
+                cast(list[Any], data),
                 debug_name,
                 throw_on_unhandled=throw_on_unhandled,
                 raw_storage_mode=raw_storage_mode,
@@ -230,26 +317,26 @@ def _deserialize(
 
 
 def _deserialize_list(
-    class_reference,
-    list_data,
-    debug_name,
+    class_reference: type[T],
+    list_data: list[Any],
+    debug_name: str,
     *,
     throw_on_unhandled: bool,
     raw_storage_mode: RawStorageMode,
-):
-    if not isinstance(list_data, list):
-        raise DeserializeException(
-            f"Cannot deserialize '{type(list_data)}' as a list for {debug_name}."
-        )
-
+) -> T:
     if not is_list(class_reference):
         raise DeserializeException(
             f"Cannot deserialize a list to '{class_reference}' for {debug_name}"
         )
 
+    if not isinstance(list_data, list):  # pyright: ignore[reportUnnecessaryIsInstance]
+        raise DeserializeException(
+            f"Cannot deserialize '{type(list_data)}' as a list for {debug_name}."
+        )
+
     list_content_type_value = list_content_type(class_reference, debug_name)
 
-    output = []
+    output: list[Any] = []
 
     for index, item in enumerate(list_data):
         deserialized = _deserialize(
@@ -261,12 +348,111 @@ def _deserialize_list(
         )
         output.append(deserialized)
 
-    return output
+    return cast(T, output)
+
+
+def _deserialize_set(
+    class_reference: type[T],
+    list_data: list[Any],
+    debug_name: str,
+    *,
+    throw_on_unhandled: bool,
+    raw_storage_mode: RawStorageMode,
+) -> T:
+    if not is_set(class_reference):
+        raise DeserializeException(
+            f"Cannot deserialize a list to '{class_reference}' for {debug_name}"
+        )
+
+    if not isinstance(list_data, list):  # pyright: ignore[reportUnnecessaryIsInstance]
+        raise DeserializeException(
+            f"Cannot deserialize '{type(list_data)}' as a set for {debug_name}."
+        )
+
+    set_content_type_value = set_content_type(class_reference, debug_name)
+
+    output: set[Any] = set()
+
+    for index, item in enumerate(list_data):
+        deserialized = _deserialize(
+            set_content_type_value,
+            item,
+            f"{debug_name}[{index}]",
+            throw_on_unhandled=throw_on_unhandled,
+            raw_storage_mode=raw_storage_mode.child_mode(),
+        )
+        output.add(deserialized)
+
+    return cast(T, output)
+
+
+def _deserialize_tuple(
+    class_reference: type[T],
+    list_data: list[Any],
+    debug_name: str,
+    *,
+    throw_on_unhandled: bool,
+    raw_storage_mode: RawStorageMode,
+) -> T:
+    if not is_tuple(class_reference):
+        raise DeserializeException(
+            f"Cannot deserialize a list to '{class_reference}' for {debug_name}"
+        )
+
+    if not isinstance(list_data, list):  # pyright: ignore[reportUnnecessaryIsInstance]
+        raise DeserializeException(
+            f"Cannot deserialize '{type(list_data)}' as a tuple for {debug_name}."
+        )
+
+    tuple_types = tuple_content_types(class_reference, debug_name)
+
+    # Handle untyped tuple
+    if len(tuple_types) == 0:
+        return cast(T, tuple(list_data))
+
+    # Handle variable-length tuple (e.g., tuple[int, ...])
+    if len(tuple_types) == 2 and tuple_types[1] is Ellipsis:
+        element_type = tuple_types[0]
+        output: list[Any] = []
+        for index, item in enumerate(list_data):
+            deserialized = _deserialize(
+                element_type,
+                item,
+                f"{debug_name}[{index}]",
+                throw_on_unhandled=throw_on_unhandled,
+                raw_storage_mode=raw_storage_mode.child_mode(),
+            )
+            output.append(deserialized)
+        return cast(T, tuple(output))
+
+    # Handle fixed-length tuple (e.g., tuple[int, str, bool])
+    if len(list_data) != len(tuple_types):
+        raise DeserializeException(
+            f"Cannot deserialize list of length {len(list_data)} to tuple of length {len(tuple_types)} for {debug_name}"
+        )
+
+    output = []
+    for index, (item, expected_type) in enumerate(zip(list_data, tuple_types)):
+        deserialized = _deserialize(
+            expected_type,
+            item,
+            f"{debug_name}[{index}]",
+            throw_on_unhandled=throw_on_unhandled,
+            raw_storage_mode=raw_storage_mode.child_mode(),
+        )
+        output.append(deserialized)
+
+    return cast(T, tuple(output))
 
 
 def _deserialize_dict(
-    class_reference, data, debug_name, *, throw_on_unhandled: bool, raw_storage_mode: RawStorageMode
-):
+    class_reference: type[T],
+    data: dict[Any, Any],
+    debug_name: str,
+    *,
+    throw_on_unhandled: bool,
+    raw_storage_mode: RawStorageMode,
+) -> T:
     """Deserialize a dictionary to a Python object."""
 
     # Check if we are doing a straightforward dictionary parse first, or if it
@@ -274,7 +460,7 @@ def _deserialize_dict(
 
     remaining_properties = set(data.keys())
 
-    if not isinstance(data, dict):
+    if not isinstance(data, dict):  # pyright: ignore[reportUnnecessaryIsInstance]
         raise DeserializeException(
             f"Data was not dict for instance: {class_reference} for {debug_name}"
         )
@@ -282,9 +468,9 @@ def _deserialize_dict(
     if is_dict(class_reference):
         if class_reference is dict:
             # If types of dictionary entries are not defined, do not deserialize
-            return data
+            return cast(T, data)
         key_type, value_type = dict_content_types(class_reference, debug_name)
-        result = {}
+        result: dict[Any, Any] = {}
 
         for dict_key, dict_value in data.items():
             if key_type != Any and not isinstance(dict_key, key_type):
@@ -307,93 +493,107 @@ def _deserialize_dict(
                 f"The following field was unhandled: {list(remaining_properties)[0]} for {debug_name}"
             )
 
-        return result
+        return cast(T, result)
 
     # It wasn't a straight forward dictionary, so we are in deserialize mode
 
-    class_instance = None
+    # Use metadata cache for performance
+    metadata = get_class_metadata(class_reference)
 
-    class_reference_downcast_field = _get_downcast_field(class_reference)
-    if class_reference_downcast_field:
-        downcast_value = data[class_reference_downcast_field]
+    # Handle downcasting
+    if metadata.downcast_field:
+        downcast_value = data[metadata.downcast_field]
         new_reference = _get_downcast_class(class_reference, downcast_value)
         if new_reference is None:
-            if _allows_downcast_fallback(class_reference):
-                return _deserialize(
-                    Dict[Any, Any],
+            if metadata.allows_downcast_fallback:
+                result_dict = _deserialize(
+                    dict[Any, Any],
                     data,
                     debug_name,
                     throw_on_unhandled=throw_on_unhandled,
                     raw_storage_mode=raw_storage_mode.child_mode(),
                 )
+                return cast(T, result_dict)
             raise UndefinedDowncastException(
                 f"Could not find subclass of {class_reference} with downcast identifier '{downcast_value}' for {debug_name}"
             )
-        class_reference = new_reference
+        # Update class reference and get new metadata
+        class_reference = cast(type[T], new_reference)
+        metadata = get_class_metadata(class_reference)
 
     try:
-        class_instance = class_reference.__new__(class_reference)
+        class_instance: T = class_reference.__new__(class_reference)
     except TypeError as ex:
-        raise DeserializeException(f"Could not create instance of {class_reference} for {debug_name}") from ex
+        raise DeserializeException(
+            f"Could not create instance of {class_reference} for {debug_name}"
+        ) from ex
 
-    handled_fields = set()
+    handled_fields: set[str] = set()
 
-    hints = typing.get_type_hints(class_reference)
-
-    if len(hints) == 0:
+    # Check if we have type hints (using cached hints from metadata)
+    if len(metadata.hints) == 0:
         raise DeserializeException(
             f"Could not deserialize {data} into {class_reference} due to lack of type hints ({debug_name})"
         )
 
-    for attribute_name, attribute_type in hints.items():
-        if _should_ignore(class_reference, attribute_name):
+    # Process fields using cached metadata
+    for attribute_name, field_meta in metadata.fields.items():
+        # Skip ignored fields
+        if field_meta.ignore:
             continue
 
-        property_key = _get_key(class_reference, attribute_name)
-        parser_function = _get_parser(class_reference, property_key)
-
-        if is_classvar(attribute_type):
-            if property_key in data:
+        # Skip ClassVars
+        if field_meta.is_classvar:
+            if field_meta.key in data:
                 raise DeserializeException(
                     f"ClassVars cannot be set: {debug_name}.{attribute_name}"
                 )
             continue
 
-        if _uses_auto_snake(class_reference) and attribute_name.lower() != attribute_name:
+        # Check auto_snake property naming
+        if metadata.auto_snake and attribute_name.lower() != attribute_name:
             raise DeserializeException(
                 f"When using auto_snake, all properties must be snake cased. Error on: {debug_name}.{attribute_name}"
             )
 
         using_default = False
+        property_value: Any = None
+        deserialized_value: Any = None
 
-        if property_key in data:
-            value = data[property_key]
-            handled_fields.add(property_key)
-            property_value = parser_function(value)
-        elif _uses_auto_snake(class_reference) and camel_case(property_key) in data:
-            value = data[camel_case(property_key)]
-            handled_fields.add(camel_case(property_key))
-            property_value = parser_function(value)
-        elif _uses_auto_snake(class_reference) and pascal_case(property_key) in data:
-            value = data[pascal_case(property_key)]
-            handled_fields.add(pascal_case(property_key))
-            property_value = parser_function(value)
+        # Look up value in data (using pre-computed keys for auto_snake)
+        if field_meta.key in data:
+            value = data[field_meta.key]
+            handled_fields.add(field_meta.key)
+            property_value = field_meta.parser(value)
+        elif metadata.auto_snake and field_meta.camel_key and field_meta.camel_key in data:
+            value = data[field_meta.camel_key]
+            handled_fields.add(field_meta.camel_key)
+            property_value = field_meta.parser(value)
+        elif metadata.auto_snake and field_meta.pascal_key and field_meta.pascal_key in data:
+            value = data[field_meta.pascal_key]
+            handled_fields.add(field_meta.pascal_key)
+            property_value = field_meta.parser(value)
         else:
-            if _has_default(class_reference, attribute_name):
-                deserialized_value = _get_default(class_reference, attribute_name)
+            # Value not in data - check for default or None
+            if field_meta.has_default:
+                deserialized_value = field_meta.default_value
                 using_default = True
             else:
-                if not is_union(attribute_type) or type(None) not in union_types(
-                    attribute_type, debug_name
+                # Check if None is acceptable (Union with None)
+                if (
+                    field_meta.is_union
+                    and field_meta.union_types
+                    and type(None) in field_meta.union_types
                 ):
+                    property_value = field_meta.parser(None)
+                else:
                     raise DeserializeException(
                         f"Unexpected missing value for: {debug_name}.{attribute_name}"
                     )
-                property_value = parser_function(None)
 
         if not using_default:
             deserialized_value = _deserialize(
-                attribute_type,
+                field_meta.type,
                 property_value,
                 f"{debug_name}.{attribute_name}",
                 throw_on_unhandled=throw_on_unhandled,
