@@ -1,10 +1,14 @@
 """Decorators used for adding functionality to the library."""
 
+from typing import Any, Callable, TypeVar
 
-def auto_snake():
+T = TypeVar("T")
+
+
+def auto_snake() -> Callable[[type[T]], type[T]]:
     """A decorator function for marking classes as those which should be auto-snaked."""
 
-    def store(class_reference):
+    def store(class_reference: type[T]) -> type[T]:
         """Store the allowance flag."""
         setattr(class_reference, "__deserialize_auto_snake__", True)
         return class_reference
@@ -12,6 +16,6 @@ def auto_snake():
     return store
 
 
-def _uses_auto_snake(super_class):
+def _uses_auto_snake(super_class: type[Any]) -> bool:
     """Get the whether auto-snake is in use or not"""
     return getattr(super_class, "__deserialize_auto_snake__", False)
